﻿using System;

namespace Ruru.Game
{
    /// <summary>
    /// 
    /// </summary>
    public class Pitcher
        : Ruru.Game.IBaseBallPlayer
    {
        /// <summary>
        /// 공들
        /// </summary>
        private int[] ball;
        /// <summary>
        /// 랜덤시드값 (기본 37)
        /// </summary>
        private static int seed = 37;
        /// <summary>
        /// 랜덤
        /// </summary>
        private Random r;
        /// <summary>
        /// 횟수
        /// </summary>
        private int num;
        /// <summary>
        /// 게임 숫자범위
        /// </summary>
        private int range;

        /// <summary>
        /// 생성자
        /// </summary>
        /// <remarks>0~range-1 개 사이의 num 개</remarks>
        /// <param name="range">범위</param>
        /// <param name="num">갯수</param>
        public Pitcher(int range, int num)
        {
            this.range = range;
            this.num = num;
            this.Init(range, num);
        }

        /// <summary>
        /// 기본생성자
        /// </summary>
        public Pitcher()
            : this(10, 3)
        {
        }

        /// <summary>
        /// 소멸자
        /// </summary>
        ~Pitcher()
        {
        }

        /// <summary>
        /// 값을 초기화한다
        /// </summary>
        /// <param name="range">범위</param>
        /// <param name="num">갯수</param>
        public void Init(int range, int num)
        {
            ball = new int[num];
            r = new Random(DateTime.Now.Millisecond + seed++);

            for (int i = 0; i < ball.Length; i++)
                ball[i] = -1;
        }

        /// <summary>
        /// 0<= MakeOne < range 임의의 정수를 반환
        /// </summary>
        /// <returns></returns>
        public int MakeOne()
        {
            return r.Next(0, range);
        }

        /// <summary>
        /// 지정된 숫자가 맴버 배열에 값이 있는가 없는가를 체크
        /// </summary>
        /// <param name="n">입력 수</param>
        /// <returns>값이 있는가 없는가</returns>
        public bool HasSame(int n)
        {
            bool hasS = false;

            for (int i = 0; i < ball.Length; i++)
            {
                if (ball[i]==n)
                {
                    hasS = true;
                    break;
                }
            }

            return hasS;
        }

        /// <summary>
        /// 야구 게임을 위한 게임 숫자가 결정된다.
        /// </summary>
        public void Make()
        {
            int temp;
            int count = 0;

            while (count != 3)
            {
                temp = this.MakeOne();
                if (!this.HasSame(temp))
                    this.ball[count++] = temp;
            }
        }

        /// <summary>
        /// 다시 만든다
        /// </summary>
        public void ReMake()
        {
            this.Init(this.range, this.num);
            this.Make();
        }

        /// <summary>
        /// 다시만들면서 범위와 수를 지정한다.
        /// </summary>
        /// <param name="range">범위</param>
        /// <param name="num">갯수</param>
        public void ReMake(int range, int num)
        {
            this.Init(range, num);
            this.Make();
        }

        /// <summary>
        /// int[] ball 에 대한 프로퍼티
        /// </summary>
        public int[] Ball
        {
            get { return this.ball; }
        }
    }
}
